"""
Flux employs a basic data model built from basic data types.

The data model consists of tables, records, columns.
"""
import codecs
import csv
from http.client import HTTPResponse
from json import JSONEncoder
from typing import List, Iterator
from influxdb_client.rest import _UTF_8_encoding


class FluxStructure:
    """The data model consists of tables, records, columns."""

    pass


class FluxStructureEncoder(JSONEncoder):
    """The FluxStructure encoder to encode query results to JSON."""

    def default(self, obj):
        """Return serializable objects for JSONEncoder."""
        import datetime
        if isinstance(obj, FluxStructure):
            return obj.__dict__
        elif isinstance(obj, (datetime.datetime, datetime.date)):
            return obj.isoformat()
        return super().default(obj)


class FluxTable(FluxStructure):
    """
    A table is set of records with a common set of columns and a group key.

    The table can be serialized into JSON by::

        import json
        from influxdb_client.client.flux_table import FluxStructureEncoder

        output = json.dumps(tables, cls=FluxStructureEncoder, indent=2)
        print(output)

    """

    def __init__(self) -> None:
        """Initialize defaults."""
        self.columns = []
        self.records = []

    def get_group_key(self):
        """
        Group key is a list of columns.

        A table’s group key denotes which subset of the entire dataset is assigned to the table.
        """
        return list(filter(lambda column: (column.group is True), self.columns))

    def __str__(self):
        """Return formatted output."""
        cls_name = type(self).__name__
        return cls_name + "() columns: " + str(len(self.columns)) + ", records: " + str(len(self.records))

    def __repr__(self):
        """Format for inspection."""
        return f"<{type(self).__name__}: {len(self.columns)} columns, {len(self.records)} records>"

    def __iter__(self):
        """Iterate over records."""
        return iter(self.records)


class FluxColumn(FluxStructure):
    """A column has a label and a data type."""

    def __init__(self, index=None, label=None, data_type=None, group=None, default_value=None) -> None:
        """Initialize defaults."""
        self.default_value = default_value
        self.group = group
        self.data_type = data_type
        self.label = label
        self.index = index

    def __repr__(self):
        """Format for inspection."""
        fields = [repr(self.index)] + [
            f'{name}={getattr(self, name)!r}' for name in (
                'label', 'data_type', 'group', 'default_value'
            ) if getattr(self, name) is not None
        ]
        return f"{type(self).__name__}({', '.join(fields)})"


class FluxRecord(FluxStructure):
    """A record is a tuple of named values and is represented using an object type."""

    def __init__(self, table, values=None) -> None:
        """Initialize defaults."""
        if values is None:
            values = {}
        self.table = table
        self.values = values
        self.row = []

    def get_start(self):
        """Get '_start' value."""
        return self["_start"]

    def get_stop(self):
        """Get '_stop' value."""
        return self["_stop"]

    def get_time(self):
        """Get timestamp."""
        return self["_time"]

    def get_value(self):
        """Get field value."""
        return self["_value"]

    def get_field(self):
        """Get field name."""
        return self["_field"]

    def get_measurement(self):
        """Get measurement name."""
        return self["_measurement"]

    def __getitem__(self, key):
        """Get value by key."""
        return self.values.__getitem__(key)

    def __setitem__(self, key, value):
        """Set value with key and value."""
        return self.values.__setitem__(key, value)

    def __str__(self):
        """Return formatted output."""
        cls_name = type(self).__name__
        return cls_name + "() table: " + str(self.table) + ", " + str(self.values)

    def __repr__(self):
        """Format for inspection."""
        return f"<{type(self).__name__}: field={self.values.get('_field')}, value={self.values.get('_value')}>"


class TableList(List[FluxTable]):
    """:class:`~influxdb_client.client.flux_table.FluxTable` list with additionally functional to better handle of query result."""  # noqa: E501

    def to_values(self, columns: List['str'] = None) -> List[List[object]]:
        """
        Serialize query results to a flattened list of values.

        :param columns: if not ``None`` then only specified columns are presented in results
        :return: :class:`~list` of values

        Output example:

        .. code-block:: python

            [
                ['New York', datetime.datetime(2022, 6, 7, 11, 3, 22, 917593, tzinfo=tzutc()), 24.3],
                ['Prague', datetime.datetime(2022, 6, 7, 11, 3, 22, 917593, tzinfo=tzutc()), 25.3],
                ...
            ]

        Configure required columns:

        .. code-block:: python

            from influxdb_client import InfluxDBClient

                with InfluxDBClient(url="http://localhost:8086", token="my-token", org="my-org") as client:

                # Query: using Table structure
                tables = client.query_api().query('from(bucket:"my-bucket") |> range(start: -10m)')

                # Serialize to values
                output = tables.to_values(columns=['location', '_time', '_value'])
                print(output)
        """

        def filter_values(record):
            if columns is not None:
                return [record.values.get(k) for k in columns]
            return record.values.values()

        return self._to_values(filter_values)

    def to_json(self, columns: List['str'] = None, **kwargs) -> str:
        """
        Serialize query results to a JSON formatted :class:`~str`.

        :param columns: if not ``None`` then only specified columns are presented in results
        :return: :class:`~str`

        The query results is flattened to array:

        .. code-block:: javascript

            [
                {
                    "_measurement": "mem",
                    "_start": "2021-06-23T06:50:11.897825+00:00",
                    "_stop": "2021-06-25T06:50:11.897825+00:00",
                    "_time": "2020-02-27T16:20:00.897825+00:00",
                    "region": "north",
                     "_field": "usage",
                    "_value": 15
                },
                {
                    "_measurement": "mem",
                    "_start": "2021-06-23T06:50:11.897825+00:00",
                    "_stop": "2021-06-25T06:50:11.897825+00:00",
                    "_time": "2020-02-27T16:20:01.897825+00:00",
                    "region": "west",
                     "_field": "usage",
                    "_value": 10
                },
                ...
            ]

        The JSON format could be configured via ``**kwargs`` arguments:

        .. code-block:: python

            from influxdb_client import InfluxDBClient

            with InfluxDBClient(url="http://localhost:8086", token="my-token", org="my-org") as client:

                # Query: using Table structure
                tables = client.query_api().query('from(bucket:"my-bucket") |> range(start: -10m)')

                # Serialize to JSON
                output = tables.to_json(indent=5)
                print(output)

        For all available options see - `json.dump <https://docs.python.org/3/library/json.html#json.dump>`_.
        """
        if 'indent' not in kwargs:
            kwargs['indent'] = 2

        def filter_values(record):
            if columns is not None:
                return {k: v for (k, v) in record.values.items() if k in columns}
            return record.values

        import json
        return json.dumps(self._to_values(filter_values), cls=FluxStructureEncoder, **kwargs)

    def _to_values(self, mapping):
        return [mapping(record) for table in self for record in table.records]


class CSVIterator(Iterator[List[str]]):
    """:class:`Iterator[List[str]]` with additionally functional to better handle of query result."""

    def __init__(self, response: HTTPResponse) -> None:
        """Initialize ``csv.reader``."""
        self.delegate = csv.reader(codecs.iterdecode(response, _UTF_8_encoding))

    def __iter__(self):
        """Return an iterator object."""
        return self

    def __next__(self):
        """Retrieve the next item from the iterator."""
        row = self.delegate.__next__()
        while not row:
            row = self.delegate.__next__()
        return row

    def to_values(self) -> List[List[str]]:
        """
        Serialize query results to a flattened list of values.

        :return: :class:`~list` of values

        Output example:

        .. code-block:: python

            [
                ['New York', '2022-06-14T08:00:51.749072045Z', '24.3'],
                ['Prague', '2022-06-14T08:00:51.749072045Z', '25.3'],
                ...
            ]
        """
        return list(self.__iter__())



import pytest

from influxdb_client.rest import ApiException
from tests.base_test import BaseTest


class TestLabelsApi(BaseTest):

    def setUp(self) -> None:
        super(TestLabelsApi, self).setUp()
        self.organization = self.find_my_org()
        labels = self.labels_api.find_labels()
        for label in labels:
            self.labels_api.delete_label(label)

    def test_create_label_unicode(self):
        name = self.generate_name("Cool resource 🍺")
        properties = {"color": "red 🍺", "source": "remote api"}

        label = self.labels_api.create_label(name, self.organization.id, properties)
        self.assertIsNotNone(label)
        self.assertIsNotNone(label.id)
        self.assertEqual(label.name, name)
        self.assertEqual(label.properties["color"], "red 🍺")
        self.assertEqual(label.properties["source"], "remote api")

    def test_create_label_empty_properties(self):
        name = self.generate_name("Cool resource")
        label = self.labels_api.create_label(name=name, org_id=self.organization.id)
        self.assertIsNotNone(label)
        self.assertIsNotNone(label.id)
        self.assertEqual(label.name, name)
        self.assertEqual(label.properties, None)

    def test_update_label(self):
        label = self.labels_api.create_label(self.generate_name("Cool Resource"), org_id=self.organization.id)
        self.assertEqual(label.properties, None)

        label.properties = {"color": "blue"}

        label = self.labels_api.update_label(label)

        self.assertEqual(label.properties["color"], "blue")
        self.assertEqual(len(label.properties), 1)

        label.properties["type"] = "free"
        label = self.labels_api.update_label(label)
        self.assertEqual(label.properties["color"], "blue")
        self.assertEqual(label.properties["type"], "free")
        self.assertEqual(len(label.properties), 2)

        label.properties["type"] = "paid"
        label.properties["color"] = ""

        label = self.labels_api.update_label(label)
        self.assertEqual(label.properties["type"], "paid")
        self.assertEqual(len(label.properties), 1)

    def test_delete_label(self):
        created_label = self.labels_api.create_label(self.generate_name("Cool Resource"), org_id=self.organization.id)
        self.assertIsNotNone(created_label)

        found_label = self.labels_api.find_label_by_id(label_id=created_label.id)
        self.assertIsNotNone(found_label)

        self.labels_api.delete_label(created_label)

        with pytest.raises(ApiException) as e:
            assert self.labels_api.find_label_by_id(label_id=created_label.id)
        assert "label not found" in e.value.body

    def test_clone_label(self):
        name = self.generate_name("cloned")

        properties = {
            "color": "green",
            "location": "west"
        }

        label = self.labels_api.create_label(name, self.organization.id, properties)

        cloned = self.labels_api.clone_label(cloned_name=name+"_clone", label=label)
        self.assertEqual(cloned.name, name+"_clone")
        self.assertEqual(cloned.properties, properties)

    def test_find_label_by_id(self):
        label = self.labels_api.create_label(self.generate_name("Cool Resource"), org_id=self.organization.id)

        label_by_id = self.labels_api.find_label_by_id(label_id=label.id)

        self.assertEqual(label, label_by_id)

    def test_find_label_by_id_none(self):
        with pytest.raises(ApiException) as e:
            assert self.labels_api.find_label_by_id(label_id="020f755c3c082000")
        assert "label not found" in e.value.body

    def test_find_labels(self):
        size = len(self.labels_api.find_labels())
        self.labels_api.create_label(self.generate_name("Cool Resource"), org_id=self.organization.id)

        labels = self.labels_api.find_labels()
        self.assertEqual(len(labels), size + 1)

    def test_find_label_by_org(self):
        organization = self.organizations_api.create_organization(self.generate_name("org"))
        labels = self.labels_api.find_label_by_org(org_id=organization.id)
        self.assertEqual(len(labels), 0)

        self.labels_api.create_label(self.generate_name("Cool Resource"), org_id=organization.id)
        labels = self.labels_api.find_label_by_org(org_id=organization.id)
        self.assertEqual(len(labels), 1)

        self.organizations_api.delete_organization(organization.id)

        with pytest.raises(ApiException) as e:
            assert self.organizations_api.find_organization(org_id=organization.id)
        assert "organization not found" in e.value.body



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class PostBucketRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'org_id': 'str',
        'name': 'str',
        'description': 'str',
        'rp': 'str',
        'retention_rules': 'list[BucketRetentionRules]',
        'schema_type': 'SchemaType'
    }

    attribute_map = {
        'org_id': 'orgID',
        'name': 'name',
        'description': 'description',
        'rp': 'rp',
        'retention_rules': 'retentionRules',
        'schema_type': 'schemaType'
    }

    def __init__(self, org_id=None, name=None, description=None, rp='0', retention_rules=None, schema_type=None):  # noqa: E501,D401,D403
        """PostBucketRequest - a model defined in OpenAPI."""  # noqa: E501
        self._org_id = None
        self._name = None
        self._description = None
        self._rp = None
        self._retention_rules = None
        self._schema_type = None
        self.discriminator = None

        self.org_id = org_id
        self.name = name
        if description is not None:
            self.description = description
        if rp is not None:
            self.rp = rp
        if retention_rules is not None:
            self.retention_rules = retention_rules
        if schema_type is not None:
            self.schema_type = schema_type

    @property
    def org_id(self):
        """Get the org_id of this PostBucketRequest.

        The organization ID. Specifies the organization that owns the bucket.

        :return: The org_id of this PostBucketRequest.
        :rtype: str
        """  # noqa: E501
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Set the org_id of this PostBucketRequest.

        The organization ID. Specifies the organization that owns the bucket.

        :param org_id: The org_id of this PostBucketRequest.
        :type: str
        """  # noqa: E501
        if org_id is None:
            raise ValueError("Invalid value for `org_id`, must not be `None`")  # noqa: E501
        self._org_id = org_id

    @property
    def name(self):
        """Get the name of this PostBucketRequest.

        The bucket name.

        :return: The name of this PostBucketRequest.
        :rtype: str
        """  # noqa: E501
        return self._name

    @name.setter
    def name(self, name):
        """Set the name of this PostBucketRequest.

        The bucket name.

        :param name: The name of this PostBucketRequest.
        :type: str
        """  # noqa: E501
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501
        self._name = name

    @property
    def description(self):
        """Get the description of this PostBucketRequest.

        A description of the bucket.

        :return: The description of this PostBucketRequest.
        :rtype: str
        """  # noqa: E501
        return self._description

    @description.setter
    def description(self, description):
        """Set the description of this PostBucketRequest.

        A description of the bucket.

        :param description: The description of this PostBucketRequest.
        :type: str
        """  # noqa: E501
        self._description = description

    @property
    def rp(self):
        """Get the rp of this PostBucketRequest.

        The retention policy for the bucket. For InfluxDB 1.x, specifies the duration of time that each data point in the retention policy persists.  If you need compatibility with InfluxDB 1.x, specify a value for the `rp` property; otherwise, see the `retentionRules` property.  [Retention policy](https://docs.influxdata.com/influxdb/v1.8/concepts/glossary/#retention-policy-rp) is an InfluxDB 1.x concept. The InfluxDB 2.x and Cloud equivalent is [retention period](https://docs.influxdata.com/influxdb/latest/reference/glossary/#retention-period). The InfluxDB `/api/v2` API uses `RetentionRules` to configure the retention period.

        :return: The rp of this PostBucketRequest.
        :rtype: str
        """  # noqa: E501
        return self._rp

    @rp.setter
    def rp(self, rp):
        """Set the rp of this PostBucketRequest.

        The retention policy for the bucket. For InfluxDB 1.x, specifies the duration of time that each data point in the retention policy persists.  If you need compatibility with InfluxDB 1.x, specify a value for the `rp` property; otherwise, see the `retentionRules` property.  [Retention policy](https://docs.influxdata.com/influxdb/v1.8/concepts/glossary/#retention-policy-rp) is an InfluxDB 1.x concept. The InfluxDB 2.x and Cloud equivalent is [retention period](https://docs.influxdata.com/influxdb/latest/reference/glossary/#retention-period). The InfluxDB `/api/v2` API uses `RetentionRules` to configure the retention period.

        :param rp: The rp of this PostBucketRequest.
        :type: str
        """  # noqa: E501
        self._rp = rp

    @property
    def retention_rules(self):
        """Get the retention_rules of this PostBucketRequest.

        Retention rules to expire or retain data. The InfluxDB `/api/v2` API uses `RetentionRules` to configure the [retention period](https://docs.influxdata.com/influxdb/latest/reference/glossary/#retention-period).  #### InfluxDB Cloud  - `retentionRules` is required.  #### InfluxDB OSS  - `retentionRules` isn't required.

        :return: The retention_rules of this PostBucketRequest.
        :rtype: list[BucketRetentionRules]
        """  # noqa: E501
        return self._retention_rules

    @retention_rules.setter
    def retention_rules(self, retention_rules):
        """Set the retention_rules of this PostBucketRequest.

        Retention rules to expire or retain data. The InfluxDB `/api/v2` API uses `RetentionRules` to configure the [retention period](https://docs.influxdata.com/influxdb/latest/reference/glossary/#retention-period).  #### InfluxDB Cloud  - `retentionRules` is required.  #### InfluxDB OSS  - `retentionRules` isn't required.

        :param retention_rules: The retention_rules of this PostBucketRequest.
        :type: list[BucketRetentionRules]
        """  # noqa: E501
        self._retention_rules = retention_rules

    @property
    def schema_type(self):
        """Get the schema_type of this PostBucketRequest.

        :return: The schema_type of this PostBucketRequest.
        :rtype: SchemaType
        """  # noqa: E501
        return self._schema_type

    @schema_type.setter
    def schema_type(self, schema_type):
        """Set the schema_type of this PostBucketRequest.

        :param schema_type: The schema_type of this PostBucketRequest.
        :type: SchemaType
        """  # noqa: E501
        self._schema_type = schema_type

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, PostBucketRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other



import json
import unittest

import httpretty
import pytest

from influxdb_client import InfluxDBClient, BucketSchemasService
from influxdb_client.client.warnings import CloudOnlyWarning


class Warnings(unittest.TestCase):

    def setUp(self) -> None:
        httpretty.enable()
        httpretty.reset()

    def tearDown(self) -> None:
        httpretty.disable()

    def test_cloud_only_warning(self):
        httpretty.register_uri(httpretty.GET, uri="http://localhost/ping",
                               status=200, body="{}", adding_headers={'X-Influxdb-Build': 'OSS'})
        httpretty.register_uri(httpretty.GET, uri="http://localhost/api/v2/buckets/01010101/schema/measurements",
                               status=200, body=json.dumps({'measurementSchemas': []}))

        with pytest.warns(CloudOnlyWarning) as warnings:
            with InfluxDBClient(url="http://localhost", token="my-token", org="my-org") as client:
                service = BucketSchemasService(api_client=client.api_client)
                service.get_measurement_schemas(bucket_id="01010101")
        warnings = [w for w in warnings if w.category == CloudOnlyWarning]
        self.assertEqual(1, len(warnings))



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

from influxdb_client.service._base_service import _BaseService


class SigninService(_BaseService):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):  # noqa: E501,D401,D403
        """SigninService - a operation defined in OpenAPI."""
        super().__init__(api_client)

    def post_signin(self, **kwargs):  # noqa: E501,D401,D403
        """Create a user session..

        Authenticates [Basic authentication credentials](#section/Authentication/BasicAuthentication) for a [user](https://docs.influxdata.com/influxdb/latest/reference/glossary/#user), and then, if successful, generates a user session.  To authenticate a user, pass the HTTP `Authorization` header with the `Basic` scheme and the base64-encoded username and password. For syntax and more information, see [Basic Authentication](#section/Authentication/BasicAuthentication) for syntax and more information.  If authentication is successful, InfluxDB creates a new session for the user and then returns the session cookie in the `Set-Cookie` response header.  InfluxDB stores user sessions in memory only. They expire within ten minutes and during restarts of the InfluxDB instance.  #### User sessions with authorizations  - In InfluxDB Cloud, a user session inherits all the user's permissions for   the organization. - In InfluxDB OSS, a user session inherits all the user's permissions for all   the organizations that the user belongs to.  #### Related endpoints  - [Signout](#tag/Signout)
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_signin(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str zap_trace_span: OpenTracing span context
        :param str authorization: An auth credential for the Basic scheme
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_signin_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.post_signin_with_http_info(**kwargs)  # noqa: E501
            return data

    def post_signin_with_http_info(self, **kwargs):  # noqa: E501,D401,D403
        """Create a user session..

        Authenticates [Basic authentication credentials](#section/Authentication/BasicAuthentication) for a [user](https://docs.influxdata.com/influxdb/latest/reference/glossary/#user), and then, if successful, generates a user session.  To authenticate a user, pass the HTTP `Authorization` header with the `Basic` scheme and the base64-encoded username and password. For syntax and more information, see [Basic Authentication](#section/Authentication/BasicAuthentication) for syntax and more information.  If authentication is successful, InfluxDB creates a new session for the user and then returns the session cookie in the `Set-Cookie` response header.  InfluxDB stores user sessions in memory only. They expire within ten minutes and during restarts of the InfluxDB instance.  #### User sessions with authorizations  - In InfluxDB Cloud, a user session inherits all the user's permissions for   the organization. - In InfluxDB OSS, a user session inherits all the user's permissions for all   the organizations that the user belongs to.  #### Related endpoints  - [Signout](#tag/Signout)
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_signin_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str zap_trace_span: OpenTracing span context
        :param str authorization: An auth credential for the Basic scheme
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_signin_prepare(**kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/signin', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=['BasicAuthentication'],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def post_signin_async(self, **kwargs):  # noqa: E501,D401,D403
        """Create a user session..

        Authenticates [Basic authentication credentials](#section/Authentication/BasicAuthentication) for a [user](https://docs.influxdata.com/influxdb/latest/reference/glossary/#user), and then, if successful, generates a user session.  To authenticate a user, pass the HTTP `Authorization` header with the `Basic` scheme and the base64-encoded username and password. For syntax and more information, see [Basic Authentication](#section/Authentication/BasicAuthentication) for syntax and more information.  If authentication is successful, InfluxDB creates a new session for the user and then returns the session cookie in the `Set-Cookie` response header.  InfluxDB stores user sessions in memory only. They expire within ten minutes and during restarts of the InfluxDB instance.  #### User sessions with authorizations  - In InfluxDB Cloud, a user session inherits all the user's permissions for   the organization. - In InfluxDB OSS, a user session inherits all the user's permissions for all   the organizations that the user belongs to.  #### Related endpoints  - [Signout](#tag/Signout)
        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str zap_trace_span: OpenTracing span context
        :param str authorization: An auth credential for the Basic scheme
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_signin_prepare(**kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/signin', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=['BasicAuthentication'],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _post_signin_prepare(self, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['zap_trace_span', 'authorization']  # noqa: E501
        self._check_operation_params('post_signin', all_params, local_var_params)

        path_params = {}

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501
        if 'authorization' in local_var_params:
            header_params['Authorization'] = local_var_params['authorization']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params



# flake8: noqa

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import apis into api package
from influxdb_client.service.authorizations_service import AuthorizationsService
from influxdb_client.service.backup_service import BackupService
from influxdb_client.service.bucket_schemas_service import BucketSchemasService
from influxdb_client.service.buckets_service import BucketsService
from influxdb_client.service.cells_service import CellsService
from influxdb_client.service.checks_service import ChecksService
from influxdb_client.service.config_service import ConfigService
from influxdb_client.service.dbr_ps_service import DBRPsService
from influxdb_client.service.dashboards_service import DashboardsService
from influxdb_client.service.delete_service import DeleteService
from influxdb_client.service.health_service import HealthService
from influxdb_client.service.invokable_scripts_service import InvokableScriptsService
from influxdb_client.service.labels_service import LabelsService
from influxdb_client.service.legacy_authorizations_service import LegacyAuthorizationsService
from influxdb_client.service.metrics_service import MetricsService
from influxdb_client.service.notification_endpoints_service import NotificationEndpointsService
from influxdb_client.service.notification_rules_service import NotificationRulesService
from influxdb_client.service.organizations_service import OrganizationsService
from influxdb_client.service.ping_service import PingService
from influxdb_client.service.query_service import QueryService
from influxdb_client.service.ready_service import ReadyService
from influxdb_client.service.remote_connections_service import RemoteConnectionsService
from influxdb_client.service.replications_service import ReplicationsService
from influxdb_client.service.resources_service import ResourcesService
from influxdb_client.service.restore_service import RestoreService
from influxdb_client.service.routes_service import RoutesService
from influxdb_client.service.rules_service import RulesService
from influxdb_client.service.scraper_targets_service import ScraperTargetsService
from influxdb_client.service.secrets_service import SecretsService
from influxdb_client.service.setup_service import SetupService
from influxdb_client.service.signin_service import SigninService
from influxdb_client.service.signout_service import SignoutService
from influxdb_client.service.sources_service import SourcesService
from influxdb_client.service.tasks_service import TasksService
from influxdb_client.service.telegraf_plugins_service import TelegrafPluginsService
from influxdb_client.service.telegrafs_service import TelegrafsService
from influxdb_client.service.templates_service import TemplatesService
from influxdb_client.service.users_service import UsersService
from influxdb_client.service.variables_service import VariablesService
from influxdb_client.service.views_service import ViewsService
from influxdb_client.service.write_service import WriteService



import datetime
import unittest

import httpretty
from urllib3 import Retry

from influxdb_client import InfluxDBClient
from tests.base_test import BaseTest


class TestHealth(BaseTest):

    def setUp(self) -> None:
        super(TestHealth, self).setUp()
        self.client.api_client.configuration.debug = True

    def test_health(self):
        health = self.client.health()
        self.assertEqual(health.message, 'ready for queries and writes')
        self.assertEqual(health.status, "pass")
        self.assertEqual(health.name, "influxdb")

    def test_health_not_running_instance(self):
        client_not_running = InfluxDBClient("http://localhost:8099", token="my-token", debug=True)
        check = client_not_running.health()
        self.assertTrue("Connection refused" in check.message)
        self.assertEqual(check.status, "fail")
        self.assertEqual(check.name, "influxdb")

    def test_ready(self):
        ready = self.client.ready()
        self.assertEqual(ready.status, "ready")
        self.assertIsNotNone(ready.started)
        self.assertTrue(datetime.datetime.now(tz=ready.started.tzinfo) > ready.started)
        self.assertIsNotNone(ready.up)


class TestHealthMock(unittest.TestCase):

    def setUp(self) -> None:
        httpretty.enable()
        httpretty.reset()

        self.influxdb_client = InfluxDBClient(url="http://localhost", token="my-token")

    def tearDown(self) -> None:
        self.influxdb_client.close()
        httpretty.disable()

    def test_without_retry(self):
        httpretty.register_uri(httpretty.GET, uri="http://localhost/health", status=429,
                               adding_headers={'Retry-After': '5', 'Content-Type': 'application/json'},
                               body="{\"message\":\"Health is not working\"}")

        check = self.influxdb_client.health()
        self.assertTrue("Health is not working" in check.message, msg=check.message)
        self.assertEqual(check.status, "fail")
        self.assertEqual(check.name, "influxdb")

        self.assertEqual(1, len(httpretty.httpretty.latest_requests))

    def test_with_retry(self):

        self.influxdb_client.close()
        self.influxdb_client = InfluxDBClient(url="http://localhost", token="my-token", retries=Retry())

        httpretty.register_uri(httpretty.GET, uri="http://localhost/health", status=200,
                               adding_headers={'Content-Type': 'application/json'},
                               body="{\"message\":\"ready for queries and writes\", \"name\":\"influxdb\", \"status\":\"pass\"}")
        httpretty.register_uri(httpretty.GET, uri="http://localhost/health", status=429,
                               adding_headers={'Retry-After': '1', 'Content-Type': 'application/json'},
                               body="{\"message\":\"Health is not working\"}")

        health = self.influxdb_client.health()
        self.assertEqual(health.message, 'ready for queries and writes')
        self.assertEqual(health.status, "pass")
        self.assertEqual(health.name, "influxdb")

        self.assertEqual(2, len(httpretty.httpretty.latest_requests))



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class Axis(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'bounds': 'list[str]',
        'label': 'str',
        'prefix': 'str',
        'suffix': 'str',
        'base': 'str',
        'scale': 'AxisScale'
    }

    attribute_map = {
        'bounds': 'bounds',
        'label': 'label',
        'prefix': 'prefix',
        'suffix': 'suffix',
        'base': 'base',
        'scale': 'scale'
    }

    def __init__(self, bounds=None, label=None, prefix=None, suffix=None, base=None, scale=None):  # noqa: E501,D401,D403
        """Axis - a model defined in OpenAPI."""  # noqa: E501
        self._bounds = None
        self._label = None
        self._prefix = None
        self._suffix = None
        self._base = None
        self._scale = None
        self.discriminator = None

        if bounds is not None:
            self.bounds = bounds
        if label is not None:
            self.label = label
        if prefix is not None:
            self.prefix = prefix
        if suffix is not None:
            self.suffix = suffix
        if base is not None:
            self.base = base
        if scale is not None:
            self.scale = scale

    @property
    def bounds(self):
        """Get the bounds of this Axis.

        The extents of the axis in the form [lower, upper]. Clients determine whether bounds are inclusive or exclusive of their limits.

        :return: The bounds of this Axis.
        :rtype: list[str]
        """  # noqa: E501
        return self._bounds

    @bounds.setter
    def bounds(self, bounds):
        """Set the bounds of this Axis.

        The extents of the axis in the form [lower, upper]. Clients determine whether bounds are inclusive or exclusive of their limits.

        :param bounds: The bounds of this Axis.
        :type: list[str]
        """  # noqa: E501
        self._bounds = bounds

    @property
    def label(self):
        """Get the label of this Axis.

        Description of the axis.

        :return: The label of this Axis.
        :rtype: str
        """  # noqa: E501
        return self._label

    @label.setter
    def label(self, label):
        """Set the label of this Axis.

        Description of the axis.

        :param label: The label of this Axis.
        :type: str
        """  # noqa: E501
        self._label = label

    @property
    def prefix(self):
        """Get the prefix of this Axis.

        Label prefix for formatting axis values.

        :return: The prefix of this Axis.
        :rtype: str
        """  # noqa: E501
        return self._prefix

    @prefix.setter
    def prefix(self, prefix):
        """Set the prefix of this Axis.

        Label prefix for formatting axis values.

        :param prefix: The prefix of this Axis.
        :type: str
        """  # noqa: E501
        self._prefix = prefix

    @property
    def suffix(self):
        """Get the suffix of this Axis.

        Label suffix for formatting axis values.

        :return: The suffix of this Axis.
        :rtype: str
        """  # noqa: E501
        return self._suffix

    @suffix.setter
    def suffix(self, suffix):
        """Set the suffix of this Axis.

        Label suffix for formatting axis values.

        :param suffix: The suffix of this Axis.
        :type: str
        """  # noqa: E501
        self._suffix = suffix

    @property
    def base(self):
        """Get the base of this Axis.

        Radix for formatting axis values.

        :return: The base of this Axis.
        :rtype: str
        """  # noqa: E501
        return self._base

    @base.setter
    def base(self, base):
        """Set the base of this Axis.

        Radix for formatting axis values.

        :param base: The base of this Axis.
        :type: str
        """  # noqa: E501
        self._base = base

    @property
    def scale(self):
        """Get the scale of this Axis.

        :return: The scale of this Axis.
        :rtype: AxisScale
        """  # noqa: E501
        return self._scale

    @scale.setter
    def scale(self, scale):
        """Set the scale of this Axis.

        :param scale: The scale of this Axis.
        :type: AxisScale
        """  # noqa: E501
        self._scale = scale

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, Axis):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other



from datetime import datetime, timezone

from influxdb_client import PermissionResource, Permission, InfluxDBClient, Point
from influxdb_client.client.write_api import SYNCHRONOUS
from tests.base_test import BaseTest


class DeleteApiTest(BaseTest):

    def setUp(self) -> None:
        super(DeleteApiTest, self).setUp()
        response = self.buckets_api.find_buckets()

        for bucket in response.buckets:
            if bucket.name.endswith("_IT"):
                self.buckets_api.delete_bucket(bucket)

        self.bucket = self.create_test_bucket()
        self.organization = self.find_my_org()

        resource = PermissionResource(type="buckets", org_id=self.organization.id, id=self.bucket.id)
        read_bucket = Permission(resource=resource, action="read")
        write_bucket = Permission(resource=resource, action="write")

        authorization = self.client.authorizations_api().create_authorization(org_id=self.organization.id,
                                                                              permissions=[read_bucket, write_bucket])
        self.auth_token = authorization.token
        self.client.close()
        self.client = InfluxDBClient(url=self.host, token=self.auth_token, org=self.org)
        self.delete_api = self.client.delete_api()

    def test_delete_buckets(self):

        self._write_data()

        q = f'from(bucket:\"{self.bucket.name}\") |> range(start: 1970-01-01T00:00:00.000000001Z)'
        flux_tables = self.client.query_api().query(query=q, org=self.organization.id)
        self.assertEqual(len(flux_tables), 1)
        self.assertEqual(len(flux_tables[0].records), 12)

        start = "1970-01-01T00:00:00.000000001Z"
        stop = "1970-01-01T00:00:00.000000012Z"
        self.delete_api.delete(start, stop, "", bucket=self.bucket.id, org=self.organization.id)

        flux_tables2 = self.client.query_api().query(
            f'from(bucket:"{self.bucket.name}") |> range(start: 1970-01-01T00:00:00.000000001Z)',
            org=self.organization.id)
        self.assertEqual(len(flux_tables2), 0)

    def test_delete_buckets_by_name(self):

        self._write_data()

        q = f'from(bucket:\"{self.bucket.name}\") |> range(start: 1970-01-01T00:00:00.000000001Z)'
        flux_tables = self.client.query_api().query(query=q, org=self.organization.id)
        self.assertEqual(len(flux_tables), 1)
        self.assertEqual(len(flux_tables[0].records), 12)

        start = "1970-01-01T00:00:00.000000001Z"
        stop = "1970-01-01T00:00:00.000000012Z"
        self._delete_and_verify(start, stop, self.organization.name)

    def test_delete_org_parameters_types(self):

        orgs = [
            self.organization,
            self.organization.id,
            self.organization.name,
            None
        ]

        for org in orgs:
            self._write_data()
            self._delete_and_verify("1970-01-01T00:00:00.000000001Z", "1970-01-01T00:00:00.000000012Z", org)

    def test_start_stop_types(self):
        starts_stops = [
            ("1970-01-01T00:00:00.000000001Z", "1970-01-01T00:00:00.000000012Z"),
            (datetime(1970, 1, 1, 0, 0, 0, 0, timezone.utc), datetime(1970, 1, 1, 0, 0, 0, 1, timezone.utc)),
            (datetime(1970, 1, 1, 0, 0, 0, 0), datetime(1970, 1, 1, 0, 0, 0, 1))
        ]
        for start_stop in starts_stops:
            self._write_data()
            self._delete_and_verify(start_stop[0], start_stop[1], self.organization.name)

    def _delete_and_verify(self, start, stop, org):
        self.delete_api.delete(start, stop, "", bucket=self.bucket.name, org=org)
        flux_tables = self.client.query_api().query(
            f'from(bucket:"{self.bucket.name}") |> range(start: 1970-01-01T00:00:00.000000001Z)',
            org=self.organization.id)
        self.assertEqual(len(flux_tables), 0)

    def _write_data(self):

        write_api = self.client.write_api(write_options=SYNCHRONOUS)
        p1 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 7.0).time(1)
        write_api.write(bucket=self.bucket.name, org=self.organization.name, record=p1)

        p2 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 8.0).time(2)
        write_api.write(bucket=self.bucket.name, org=self.organization.name, record=p2)

        p3 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 9.0).time(3)
        p4 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 10.0).time(4)
        write_api.write(bucket=self.bucket.name, org=self.organization.name, record=[p3, p4])

        p5 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 11.0).time(5)
        p6 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 12.0).time(6)
        write_api.write(bucket=self.bucket.name, org=self.organization.name, record=[p5, p6])

        p7 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 8.0).time(7)
        write_api.write(bucket=self.bucket.name, org=self.organization.name, record=p7)
        p8 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 9.0).time(8)
        write_api.write(bucket=self.bucket.name, org=self.organization.name, record=p8)

        p9 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 9.0).time(9)
        p10 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 11.0).time(10)
        write_api.write(bucket=self.bucket.name, org=self.organization.name, record=[p9, p10])

        p11 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 11.0).time(11)
        p12 = Point(measurement_name="h2o").tag("location", "coyote_creek").field("watter_level", 13.0).time(12)
        write_api.write(bucket=self.bucket.name, org=self.organization.name, record=[p11, p12])





class _Page:
    def __init__(self, values, has_next, next_after):
        self.has_next = has_next
        self.values = values
        self.next_after = next_after

    @staticmethod
    def empty():
        return _Page([], False, None)

    @staticmethod
    def initial(after):
        return _Page([], True, after)


class _PageIterator:
    def __init__(self, page: _Page, get_next_page):
        self.page = page
        self.get_next_page = get_next_page

    def __iter__(self):
        return self

    def __next__(self):
        if not self.page.values:
            if self.page.has_next:
                self.page = self.get_next_page(self.page)
            if not self.page.values:
                raise StopIteration
        return self.page.values.pop(0)


class _Paginated:
    def __init__(self, paginated_getter, pluck_page_resources_from_response):
        self.paginated_getter = paginated_getter
        self.pluck_page_resources_from_response = pluck_page_resources_from_response

    def find_iter(self, **kwargs):
        """Iterate over resources with pagination.

        :key str org: The organization name.
        :key str org_id: The organization ID.
        :key str after: The last resource ID from which to seek from (but not including).
        :key int limit: the maximum number of items per page
        :return: resources iterator
        """

        def get_next_page(page: _Page):
            return self._find_next_page(page, **kwargs)

        return iter(_PageIterator(_Page.initial(kwargs.get('after')), get_next_page))

    def _find_next_page(self, page: _Page, **kwargs):
        if not page.has_next:
            return _Page.empty()

        kw_args = {**kwargs, 'after': page.next_after} if page.next_after is not None else kwargs
        response = self.paginated_getter(**kw_args)

        resources = self.pluck_page_resources_from_response(response)
        has_next = response.links.next is not None
        last_id = resources[-1].id if resources else None

        return _Page(resources, has_next, last_id)



import json
import math
import unittest
import pandas as pd
from io import BytesIO
from packaging import version

import pytest
from urllib3 import HTTPResponse

from influxdb_client.client.flux_csv_parser import FluxCsvParser, FluxSerializationMode, FluxQueryException, \
    FluxResponseMetadataMode
from influxdb_client.client.flux_table import FluxStructureEncoder, TableList


class FluxCsvParserTest(unittest.TestCase):

    def test_one_table(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,long,long,string\n" \
               "#group,false,false,true,true,true,true,true,true,false,false,false\n" \
               "#default,_result,,,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,_value2,value1,value_str\n" \
               ",,0,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,121,11,test\n"

        tables = self._parse_to_tables(data=data)
        self.assertEqual(1, tables.__len__())
        self.assertEqual(11, tables[0].columns.__len__())
        self.assertEqual(1, tables[0].records.__len__())

    def test_more_tables(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,long,long,string\n" \
               "#group,false,false,true,true,true,true,true,true,false,false,false\n" \
               "#default,_result,,,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,_value2,value1,value_str\n" \
               ",,0,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,121,11,test\n" \
               ",,1,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,B,west,484,22,test\n" \
               ",,2,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,usage_system,cpu,A,west,1444,38,test\n" \
               ",,3,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,user_usage,cpu,A,west,2401,49,test"

        tables = self._parse_to_tables(data=data)
        self.assertEqual(4, tables.__len__())
        self.assertEqual(11, tables[0].columns.__len__())
        self.assertEqual(1, tables[0].records.__len__())
        self.assertEqual(11, tables[1].columns.__len__())
        self.assertEqual(1, tables[1].records.__len__())
        self.assertEqual(11, tables[2].columns.__len__())
        self.assertEqual(1, tables[2].records.__len__())
        self.assertEqual(11, tables[3].columns.__len__())
        self.assertEqual(1, tables[3].records.__len__())

    def test_multiple_queries(self):
        data = "#datatype,string,long,string,string,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,double,string\n" \
               "#group,false,false,true,true,true,true,false,false,true\n" \
               "#default,t1,,,,,,,,\n" \
               ",result,table,_field,_measurement,_start,_stop,_time,_value,tag\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:20:00Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:21:40Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:23:20Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:25:00Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:26:40Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:28:20Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:30:00Z,2,test1\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:20:00Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:21:40Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:23:20Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:25:00Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:26:40Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:28:20Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:30:00Z,2,test2\n" \
               "\n" \
               "#datatype,string,long,string,string,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,double,string\n" \
               "#group,false,false,true,true,true,true,false,false,true\n" \
               "#default,t2,,,,,,,,\n" \
               ",result,table,_field,_measurement,_start,_stop,_time,_value,tag\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:20:00Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:21:40Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:23:20Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:25:00Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:26:40Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:28:20Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:30:00Z,2,test1\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:20:00Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:21:40Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:23:20Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:25:00Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:26:40Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:28:20Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:30:00Z,2,test2"

        tables = self._parse_to_tables(data=data)
        self.assertEqual(4, tables.__len__())
        self.assertEqual(9, tables[0].columns.__len__())
        self.assertEqual(7, tables[0].records.__len__())
        self.assertEqual(9, tables[1].columns.__len__())
        self.assertEqual(7, tables[1].records.__len__())
        self.assertEqual(9, tables[2].columns.__len__())
        self.assertEqual(7, tables[2].records.__len__())
        self.assertEqual(9, tables[3].columns.__len__())
        self.assertEqual(7, tables[3].records.__len__())

    def test_table_index_not_start_at_zero(self):
        data = "#datatype,string,long,string,string,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,double,string\n" \
               "#group,false,false,true,true,true,true,false,false,true\n" \
               "#default,t1,,,,,,,,\n" \
               ",result,table,_field,_measurement,_start,_stop,_time,_value,tag\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:20:00Z,2,test1\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:21:40Z,2,test1\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:23:20Z,2,test1\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:25:00Z,2,test1\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:26:40Z,2,test1\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:28:20Z,2,test1\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:30:00Z,2,test1\n" \
               ",,2,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:20:00Z,2,test2\n" \
               ",,2,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:21:40Z,2,test2\n" \
               ",,2,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:23:20Z,2,test2\n" \
               ",,2,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:25:00Z,2,test2\n" \
               ",,2,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:26:40Z,2,test2\n" \
               ",,2,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:28:20Z,2,test2\n" \
               ",,2,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:30:00Z,2,test2\n"

        tables = self._parse_to_tables(data=data)
        self.assertEqual(2, tables.__len__())
        self.assertEqual(9, tables[0].columns.__len__())
        self.assertEqual(7, tables[0].records.__len__())
        self.assertEqual(9, tables[1].columns.__len__())
        self.assertEqual(7, tables[1].records.__len__())

    def test_response_with_error(self):
        data = "#datatype,string,long,string,string,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,double,string\n" \
                 "#group,false,false,true,true,true,true,false,false,true\n" \
                 "#default,t1,,,,,,,,\n" \
                 ",result,table,_field,_measurement,_start,_stop,_time,_value,tag\n" \
                 ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:20:00Z,2,test1\n" \
                 ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:21:40Z,2,test1\n" \
                 ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:23:20Z,2,test1\n" \
                 ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:25:00Z,2,test1\n" \
                 "\n" \
                 "#datatype,string,string\n" \
                 "#group,true,true\n" \
                 "#default,,\n" \
                 ",error,reference\n" \
                 ",\"engine: unknown field type for value: xyz\","

        with self.assertRaises(FluxQueryException) as cm:
            self._parse_to_tables(data=data)
        exception = cm.exception

        self.assertEqual('engine: unknown field type for value: xyz', exception.message)
        self.assertEqual('', exception.reference)

    def test_ParseExportFromUserInterface(self):

        data = "#group,false,false,true,true,true,true,true,true,false,false\n" \
           + "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,double,dateTime:RFC3339\n" \
           + "#default,mean,,,,,,,,,\n" \
           + ",result,table,_start,_stop,_field,_measurement,city,location,_value,_time\n" \
           + ",,0,1754-06-26T11:30:27.613654848Z,2040-10-27T12:13:46.485Z,temperatureC,weather,London,us-midwest,30,1975-09-01T16:59:54.5Z\n" \
           + ",,1,1754-06-26T11:30:27.613654848Z,2040-10-27T12:13:46.485Z,temperatureF,weather,London,us-midwest,86,1975-09-01T16:59:54.5Z\n";

        tables = self._parse_to_tables(data=data)
        self.assertEqual(2, tables.__len__())
        self.assertEqual(1, tables[0].records.__len__())
        self.assertEqual(1, tables[1].records.__len__())
        self.assertFalse(tables[1].columns[0].group)
        self.assertFalse(tables[1].columns[1].group)
        self.assertTrue(tables[1].columns[2].group)

    def test_ParseInf(self):

        data = """#group,false,false,true,true,true,true,true,true,true,true,false,false
#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,string,string,double,double
#default,_result,,,,,,,,,,,
,result,table,_start,_stop,_field,_measurement,language,license,name,owner,le,_value
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,0,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,10,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,20,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,30,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,40,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,50,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,60,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,70,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,80,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,90,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,+Inf,15
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,-Inf,15

"""
        tables = self._parse_to_tables(data=data)
        self.assertEqual(1, tables.__len__())
        self.assertEqual(12, tables[0].records.__len__())
        self.assertEqual(math.inf, tables[0].records[10]["le"])
        self.assertEqual(-math.inf, tables[0].records[11]["le"])

    def test_to_json_by_encoder(self):
        data = "#datatype,string,long,string,string,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,double,string\n" \
               "#group,false,false,true,true,true,true,false,false,true\n" \
               "#default,_result,,,,,,,,\n" \
               ",result,table,_field,_measurement,_start,_stop,_time,_value,tag\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:20:00Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:21:40Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:23:20Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:25:00Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:26:40Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:28:20Z,2,test1\n" \
               ",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:30:00Z,2,test1\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:20:00Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:21:40Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:23:20Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:25:00Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:26:40Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:28:20Z,2,test2\n" \
               ",,1,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:30:00Z,2,test2\n"

        tables = self._parse_to_tables(data=data)
        with open('tests/query_output.json', 'r') as file:
            query_output = file.read()
            import json
            self.assertEqual(query_output, json.dumps(tables, cls=FluxStructureEncoder, indent=2))

    def test_pandas_lot_of_columns(self):
        data_types = ""
        groups = ""
        defaults = ""
        columns = ""
        values = ""
        for i in range(0, 200):
            data_types += f",long"
            groups += f",false"
            defaults += f","
            columns += f",column_{i}"
            values += f",{i}"

        data = f"#datatype,string,long,string,string,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,double,string{data_types}\n" \
               f"#group,false,false,true,true,true,true,false,false,true{groups}\n" \
               f"#default,_result,,,,,,,,{defaults}\n" \
               f",result,table,_field,_measurement,_start,_stop,_time,_value,tag{columns}\n" \
               f",,0,value,python_client_test,2010-02-27T04:48:32.752600083Z,2020-02-27T16:48:32.752600083Z,2020-02-27T16:20:00Z,2,test1{values}\n" \

        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full)
        _dataFrames = list(parser.generator())
        self.assertEqual(1, _dataFrames.__len__())

    def test_pandas_column_datatype(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,long,unsignedLong,string,boolean,double\n" \
               "#group,false,false,true,true,true,true,true,true,false,false,false,false,false\n" \
               "#default,_result,,,,,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,value1,value2,value3,value4,value5\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,121,11,test,true,6.56\n"
        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full)
        df = list(parser.generator())[0]
        self.assertEqual(13, df.dtypes.__len__())
        self.assertEqual('object', df.dtypes['result'].name)
        self.assertEqual('int64', df.dtypes['table'].name)
        self.assertIn('datetime64[ns,', df.dtypes['_start'].name)
        self.assertIn('datetime64[ns,', df.dtypes['_stop'].name)
        self.assertEqual('object', df.dtypes['_field'].name)
        self.assertEqual('object', df.dtypes['_measurement'].name)
        self.assertEqual('object', df.dtypes['host'].name)
        self.assertEqual('object', df.dtypes['region'].name)
        self.assertEqual('int64', df.dtypes['value1'].name)
        self.assertEqual('int64', df.dtypes['value2'].name)
        self.assertEqual('object', df.dtypes['value3'].name)
        self.assertEqual('bool', df.dtypes['value4'].name)
        self.assertEqual('float64', df.dtypes['value5'].name)

    def test_pandas_column_datatype_extension_types(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,long,unsignedLong,string,boolean,double\n" \
               "#group,false,false,true,true,true,true,true,true,false,false,false,false,false\n" \
               "#default,_result,,,,,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,value1,value2,value3,value4,value5\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,121,11,test,true,6.56\n"
        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full,
                             use_extension_dtypes=True)
        df = list(parser.generator())[0]
        self.assertEqual(13, df.dtypes.__len__())
        self.assertEqual('string', df.dtypes['result'].name)
        self.assertEqual('Int64', df.dtypes['table'].name)
        self.assertIn('datetime64[ns,', df.dtypes['_start'].name)
        self.assertIn('datetime64[ns,', df.dtypes['_stop'].name)
        self.assertEqual('string', df.dtypes['_field'].name)
        self.assertEqual('string', df.dtypes['_measurement'].name)
        self.assertEqual('string', df.dtypes['host'].name)
        self.assertEqual('string', df.dtypes['region'].name)
        self.assertEqual('Int64', df.dtypes['value1'].name)
        self.assertEqual('Int64', df.dtypes['value2'].name)
        self.assertEqual('string', df.dtypes['value3'].name)
        self.assertEqual('boolean', df.dtypes['value4'].name)
        self.assertEqual('Float64', df.dtypes['value5'].name)

    def test_pandas_null_bool_types(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,boolean\n" \
               "#group,false,false,true,true,true,true,true,true,false\n" \
               "#default,_result,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,value\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,true\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,\n"

        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full)
        df = list(parser.generator())[0]
        self.assertEqual('object', df.dtypes['value'].name)

    def test_pandas_null_bool_types_extension_types(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,boolean\n" \
               "#group,false,false,true,true,true,true,true,true,false\n" \
               "#default,_result,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,value\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,true\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,\n"

        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full,
                             use_extension_dtypes=True)
        df = list(parser.generator())[0]
        self.assertEqual('boolean', df.dtypes['value'].name)

    def test_pandas_null_long_types(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,long\n" \
               "#group,false,false,true,true,true,true,true,true,false\n" \
               "#default,_result,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,value\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,1\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,\n"

        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full)
        df = list(parser.generator())[0]
        self.assertEqual('float64', df.dtypes['value'].name)  # pd.NA is converted to float('nan')

    @pytest.mark.skipif(version.parse(pd.__version__).release < (2, 0), reason="numeric nullables require pandas>=2.0 to work correctly")
    def test_pandas_null_long_types_extension_types(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,long\n" \
               "#group,false,false,true,true,true,true,true,true,false\n" \
               "#default,_result,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,value\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,1\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,\n"

        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full,
                             use_extension_dtypes=True)
        df = list(parser.generator())[0]
        self.assertEqual('Int64', df.dtypes['value'].name)

    def test_pandas_null_double_types(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,double\n" \
               "#group,false,false,true,true,true,true,true,true,false\n" \
               "#default,_result,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,value\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,1\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,\n"

        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full)
        df = list(parser.generator())[0]
        self.assertEqual('float64', df.dtypes['value'].name)

    @pytest.mark.skipif(version.parse(pd.__version__).release < (2, 0), reason="numeric nullables require pandas>=2.0 to work correctly")
    def test_pandas_null_double_types_extension_types(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,double\n" \
               "#group,false,false,true,true,true,true,true,true,false\n" \
               "#default,_result,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,value\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,1\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,\n"

        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full,
                             use_extension_dtypes=True)
        df = list(parser.generator())[0]
        self.assertEqual('Float64', df.dtypes['value'].name)

    def test_pandas_null_string_types(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,string\n" \
               "#group,false,false,true,true,true,true,true,true,false\n" \
               "#default,_result,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,value\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,hi\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,\n"

        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full)
        df = list(parser.generator())[0]
        self.assertEqual('object', df.dtypes['value'].name)

    def test_pandas_null_string_types_extension_types(self):
        data = "#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,string\n" \
               "#group,false,false,true,true,true,true,true,true,false\n" \
               "#default,_result,,,,,,,,\n" \
               ",result,table,_start,_stop,_field,_measurement,host,region,value\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,hi\n" \
               ",,0,1977-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,\n"

        parser = self._parse(data=data, serialization_mode=FluxSerializationMode.dataFrame,
                             response_metadata_mode=FluxResponseMetadataMode.full,
                             use_extension_dtypes=True)
        df = list(parser.generator())[0]
        self.assertEqual('string', df.dtypes['value'].name)

    def test_parse_without_datatype(self):
        data = ",result,table,_start,_stop,_field,_measurement,host,region,_value2,value1,value_str\n" \
               ",,0,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,121,11,test\n" \
               ",,1,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,121,11,test\n"

        tables = self._parse_to_tables(data=data, response_metadata_mode=FluxResponseMetadataMode.only_names)
        self.assertEqual(2, tables.__len__())
        self.assertEqual(11, tables[0].columns.__len__())
        self.assertEqual(1, tables[0].records.__len__())
        self.assertEqual(11, tables[0].records[0].values.__len__())
        self.assertEqual('0', tables[0].records[0]['table'])
        self.assertEqual('11', tables[0].records[0]['value1'])
        self.assertEqual('west', tables[0].records[0]['region'])

    def test_parse_to_json(self):
        data = """#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,long,long,string
#group,false,false,true,true,true,true,true,true,false,false,false
#default,_result,,,,,,,,,,
,result,table,_start,_stop,_field,_measurement,host,region,_value2,value1,value_str
,,0,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,121,11,test
,,0,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,C,north,50,40,abc

#group,false,false,true,true,true,true,true,true,true,true,false,false
#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,string,string,double,double
#default,_result,,,,,,,,,,,
,result,table,_start,_stop,_field,_measurement,language,license,name,owner,le,_value
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,0,0
,,0,2021-06-23T06:50:11.897825012Z,2021-06-25T06:50:11.897825012Z,stars,github_repository,C#,MIT License,influxdb-client-csharp,influxdata,10,0

"""

        tables = self._parse_to_tables(data=data)
        parsed = json.loads(tables.to_json())
        self.assertEqual(4, parsed.__len__())

        self.assertEqual(11, parsed[0].__len__())
        self.assertEqual(11, parsed[0]['value1'])
        self.assertEqual("test", parsed[0]['value_str'])

        self.assertEqual(11, parsed[1].__len__())
        self.assertEqual(40, parsed[1]['value1'])
        self.assertEqual("abc", parsed[1]['value_str'])

        self.assertEqual(12, parsed[2].__len__())
        self.assertEqual(0, parsed[2]['le'])
        self.assertEqual("influxdata", parsed[2]['owner'])

        self.assertEqual(12, parsed[3].__len__())
        self.assertEqual(10, parsed[3]['le'])
        self.assertEqual("influxdata", parsed[3]['owner'])

    def test_parse_to_json_columns(self):
        data = """#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,long,long,string
#group,false,false,true,true,true,true,true,true,false,false,false
#default,_result,,,,,,,,,,
,result,table,_start,_stop,_field,_measurement,host,region,_value2,value1,value_str
,,0,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,121,11,test
,,0,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,C,north,50,40,abc
"""

        tables = self._parse_to_tables(data=data)
        parsed = json.loads(tables.to_json(columns=['region', 'size']))
        self.assertEqual(2, parsed.__len__())

        self.assertEqual(1, parsed[0].__len__())
        self.assertEqual("west", parsed[0]['region'])
        self.assertEqual(1, parsed[1].__len__())
        self.assertEqual("north", parsed[1]['region'])

    def test_parse_to_values(self):
        data = """#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,long,long,string
#group,false,false,true,true,true,true,true,true,false,false,false
#default,_result,,,,,,,,,,
,result,table,_start,_stop,_field,_measurement,host,region,_value2,value,value_str
,,0,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,A,west,121,11,test
,,0,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,C,north,50,40,abc

#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,string,string,string,string,long,long,string
#group,false,false,true,true,true,true,true,true,false,false,false
#default,_result,,,,,,,,,,
,result,table,_start,_stop,_field,_measurement,host,region,_value2,value,value_str
,,1,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,B,south,121,18,test
,,1,1677-09-21T00:12:43.145224192Z,2018-07-16T11:21:02.547596934Z,free,mem,D,south,50,22,abc
"""

        tables = self._parse_to_tables(data=data)
        parsed = tables.to_values(['region', 'host', 'not_exits', 'value'])
        self.assertEqual(4, parsed.__len__())

        self.assertEqual(['west', 'A', None, 11], parsed[0])
        self.assertEqual(['north', 'C', None, 40], parsed[1])
        self.assertEqual(['south', 'B', None, 18], parsed[2])
        self.assertEqual(['south', 'D', None, 22], parsed[3])

    def test_parse_duplicate_column_names(self):
        data = """#datatype,string,long,dateTime:RFC3339,dateTime:RFC3339,dateTime:RFC3339,string,string,double
#group,false,false,true,true,false,true,true,false
#default,_result,,,,,,,
,result,table,_start,_stop,_time,_measurement,location,result
,,0,2022-09-13T06:14:40.469404272Z,2022-09-13T06:24:40.469404272Z,2022-09-13T06:24:33.746Z,my_measurement,Prague,25.3
,,0,2022-09-13T06:14:40.469404272Z,2022-09-13T06:24:40.469404272Z,2022-09-13T06:24:39.299Z,my_measurement,Prague,25.3
,,0,2022-09-13T06:14:40.469404272Z,2022-09-13T06:24:40.469404272Z,2022-09-13T06:24:40.454Z,my_measurement,Prague,25.3
"""
        with pytest.warns(UserWarning) as warnings:
            tables = self._parse_to_tables(data=data)
        self.assertEqual(1, len(warnings))
        self.assertEqual(1, tables.__len__())
        self.assertEqual(8, tables[0].columns.__len__())
        self.assertEqual(3, tables[0].records.__len__())
        self.assertEqual(7, tables[0].records[0].values.__len__())
        self.assertEqual(8, tables[0].records[0].row.__len__())
        self.assertEqual(25.3, tables[0].records[0].row[7])


    @staticmethod
    def _parse_to_tables(data: str, serialization_mode=FluxSerializationMode.tables,
                         response_metadata_mode=FluxResponseMetadataMode.full) -> TableList:
        _parser = FluxCsvParserTest._parse(data, serialization_mode, response_metadata_mode=response_metadata_mode)
        list(_parser.generator())
        tables = _parser.tables
        return tables

    @staticmethod
    def _parse(data, serialization_mode, response_metadata_mode, use_extension_dtypes=False):
        fp = BytesIO(str.encode(data))
        return FluxCsvParser(response=HTTPResponse(fp, preload_content=False),
                             serialization_mode=serialization_mode, response_metadata_mode=response_metadata_mode,
                             use_extension_dtypes=use_extension_dtypes)



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class TaskUpdateRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'status': 'TaskStatusType',
        'flux': 'str',
        'name': 'str',
        'every': 'str',
        'cron': 'str',
        'offset': 'str',
        'description': 'str'
    }

    attribute_map = {
        'status': 'status',
        'flux': 'flux',
        'name': 'name',
        'every': 'every',
        'cron': 'cron',
        'offset': 'offset',
        'description': 'description'
    }

    def __init__(self, status=None, flux=None, name=None, every=None, cron=None, offset=None, description=None):  # noqa: E501,D401,D403
        """TaskUpdateRequest - a model defined in OpenAPI."""  # noqa: E501
        self._status = None
        self._flux = None
        self._name = None
        self._every = None
        self._cron = None
        self._offset = None
        self._description = None
        self.discriminator = None

        if status is not None:
            self.status = status
        if flux is not None:
            self.flux = flux
        if name is not None:
            self.name = name
        if every is not None:
            self.every = every
        if cron is not None:
            self.cron = cron
        if offset is not None:
            self.offset = offset
        if description is not None:
            self.description = description

    @property
    def status(self):
        """Get the status of this TaskUpdateRequest.

        :return: The status of this TaskUpdateRequest.
        :rtype: TaskStatusType
        """  # noqa: E501
        return self._status

    @status.setter
    def status(self, status):
        """Set the status of this TaskUpdateRequest.

        :param status: The status of this TaskUpdateRequest.
        :type: TaskStatusType
        """  # noqa: E501
        self._status = status

    @property
    def flux(self):
        """Get the flux of this TaskUpdateRequest.

        The Flux script that the task runs.

        :return: The flux of this TaskUpdateRequest.
        :rtype: str
        """  # noqa: E501
        return self._flux

    @flux.setter
    def flux(self, flux):
        """Set the flux of this TaskUpdateRequest.

        The Flux script that the task runs.

        :param flux: The flux of this TaskUpdateRequest.
        :type: str
        """  # noqa: E501
        self._flux = flux

    @property
    def name(self):
        """Get the name of this TaskUpdateRequest.

        Update the 'name' option in the flux script.

        :return: The name of this TaskUpdateRequest.
        :rtype: str
        """  # noqa: E501
        return self._name

    @name.setter
    def name(self, name):
        """Set the name of this TaskUpdateRequest.

        Update the 'name' option in the flux script.

        :param name: The name of this TaskUpdateRequest.
        :type: str
        """  # noqa: E501
        self._name = name

    @property
    def every(self):
        """Get the every of this TaskUpdateRequest.

        Update the 'every' option in the flux script.

        :return: The every of this TaskUpdateRequest.
        :rtype: str
        """  # noqa: E501
        return self._every

    @every.setter
    def every(self, every):
        """Set the every of this TaskUpdateRequest.

        Update the 'every' option in the flux script.

        :param every: The every of this TaskUpdateRequest.
        :type: str
        """  # noqa: E501
        self._every = every

    @property
    def cron(self):
        """Get the cron of this TaskUpdateRequest.

        Update the 'cron' option in the flux script.

        :return: The cron of this TaskUpdateRequest.
        :rtype: str
        """  # noqa: E501
        return self._cron

    @cron.setter
    def cron(self, cron):
        """Set the cron of this TaskUpdateRequest.

        Update the 'cron' option in the flux script.

        :param cron: The cron of this TaskUpdateRequest.
        :type: str
        """  # noqa: E501
        self._cron = cron

    @property
    def offset(self):
        """Get the offset of this TaskUpdateRequest.

        Update the 'offset' option in the flux script.

        :return: The offset of this TaskUpdateRequest.
        :rtype: str
        """  # noqa: E501
        return self._offset

    @offset.setter
    def offset(self, offset):
        """Set the offset of this TaskUpdateRequest.

        Update the 'offset' option in the flux script.

        :param offset: The offset of this TaskUpdateRequest.
        :type: str
        """  # noqa: E501
        self._offset = offset

    @property
    def description(self):
        """Get the description of this TaskUpdateRequest.

        Update the description of the task.

        :return: The description of this TaskUpdateRequest.
        :rtype: str
        """  # noqa: E501
        return self._description

    @description.setter
    def description(self, description):
        """Set the description of this TaskUpdateRequest.

        Update the description of the task.

        :param description: The description of this TaskUpdateRequest.
        :type: str
        """  # noqa: E501
        self._description = description

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, TaskUpdateRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

from influxdb_client.service._base_service import _BaseService


class ChecksService(_BaseService):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):  # noqa: E501,D401,D403
        """ChecksService - a operation defined in OpenAPI."""
        super().__init__(api_client)

    def create_check(self, post_check, **kwargs):  # noqa: E501,D401,D403
        """Add new check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_check(post_check, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param PostCheck post_check: Check to create (required)
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_check_with_http_info(post_check, **kwargs)  # noqa: E501
        else:
            (data) = self.create_check_with_http_info(post_check, **kwargs)  # noqa: E501
            return data

    def create_check_with_http_info(self, post_check, **kwargs):  # noqa: E501,D401,D403
        """Add new check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.create_check_with_http_info(post_check, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param PostCheck post_check: Check to create (required)
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._create_check_prepare(post_check, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/checks', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Check',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def create_check_async(self, post_check, **kwargs):  # noqa: E501,D401,D403
        """Add new check.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param PostCheck post_check: Check to create (required)
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._create_check_prepare(post_check, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/checks', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Check',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _create_check_prepare(self, post_check, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['post_check']  # noqa: E501
        self._check_operation_params('create_check', all_params, local_var_params)
        # verify the required parameter 'post_check' is set
        if ('post_check' not in local_var_params or
                local_var_params['post_check'] is None):
            raise ValueError("Missing the required parameter `post_check` when calling `create_check`")  # noqa: E501

        path_params = {}

        query_params = []

        header_params = {}

        body_params = None
        if 'post_check' in local_var_params:
            body_params = local_var_params['post_check']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def delete_checks_id(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_checks_id(check_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_checks_id_with_http_info(check_id, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_checks_id_with_http_info(check_id, **kwargs)  # noqa: E501
            return data

    def delete_checks_id_with_http_info(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_checks_id_with_http_info(check_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_checks_id_prepare(check_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/checks/{checkID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def delete_checks_id_async(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a check.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_checks_id_prepare(check_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/checks/{checkID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _delete_checks_id_prepare(self, check_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['check_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('delete_checks_id', all_params, local_var_params)
        # verify the required parameter 'check_id' is set
        if ('check_id' not in local_var_params or
                local_var_params['check_id'] is None):
            raise ValueError("Missing the required parameter `check_id` when calling `delete_checks_id`")  # noqa: E501

        path_params = {}
        if 'check_id' in local_var_params:
            path_params['checkID'] = local_var_params['check_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def delete_checks_id_labels_id(self, check_id, label_id, **kwargs):  # noqa: E501,D401,D403
        """Delete label from a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_checks_id_labels_id(check_id, label_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str label_id: The ID of the label to delete. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_checks_id_labels_id_with_http_info(check_id, label_id, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_checks_id_labels_id_with_http_info(check_id, label_id, **kwargs)  # noqa: E501
            return data

    def delete_checks_id_labels_id_with_http_info(self, check_id, label_id, **kwargs):  # noqa: E501,D401,D403
        """Delete label from a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_checks_id_labels_id_with_http_info(check_id, label_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str label_id: The ID of the label to delete. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_checks_id_labels_id_prepare(check_id, label_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/checks/{checkID}/labels/{labelID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def delete_checks_id_labels_id_async(self, check_id, label_id, **kwargs):  # noqa: E501,D401,D403
        """Delete label from a check.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str label_id: The ID of the label to delete. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_checks_id_labels_id_prepare(check_id, label_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/checks/{checkID}/labels/{labelID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _delete_checks_id_labels_id_prepare(self, check_id, label_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['check_id', 'label_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('delete_checks_id_labels_id', all_params, local_var_params)
        # verify the required parameter 'check_id' is set
        if ('check_id' not in local_var_params or
                local_var_params['check_id'] is None):
            raise ValueError("Missing the required parameter `check_id` when calling `delete_checks_id_labels_id`")  # noqa: E501
        # verify the required parameter 'label_id' is set
        if ('label_id' not in local_var_params or
                local_var_params['label_id'] is None):
            raise ValueError("Missing the required parameter `label_id` when calling `delete_checks_id_labels_id`")  # noqa: E501

        path_params = {}
        if 'check_id' in local_var_params:
            path_params['checkID'] = local_var_params['check_id']  # noqa: E501
        if 'label_id' in local_var_params:
            path_params['labelID'] = local_var_params['label_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def get_checks(self, org_id, **kwargs):  # noqa: E501,D401,D403
        """List all checks.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_checks(org_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str org_id: Only show checks that belong to a specific organization ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :param int offset: The offset for pagination. The number of records to skip.  For more information about pagination parameters, see [Pagination](https://docs.influxdata.com/influxdb/latest/api/#tag/Pagination).
        :param int limit: Limits the number of records returned. Default is `20`.
        :return: Checks
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_checks_with_http_info(org_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_checks_with_http_info(org_id, **kwargs)  # noqa: E501
            return data

    def get_checks_with_http_info(self, org_id, **kwargs):  # noqa: E501,D401,D403
        """List all checks.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_checks_with_http_info(org_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str org_id: Only show checks that belong to a specific organization ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :param int offset: The offset for pagination. The number of records to skip.  For more information about pagination parameters, see [Pagination](https://docs.influxdata.com/influxdb/latest/api/#tag/Pagination).
        :param int limit: Limits the number of records returned. Default is `20`.
        :return: Checks
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_checks_prepare(org_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/checks', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Checks',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def get_checks_async(self, org_id, **kwargs):  # noqa: E501,D401,D403
        """List all checks.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str org_id: Only show checks that belong to a specific organization ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :param int offset: The offset for pagination. The number of records to skip.  For more information about pagination parameters, see [Pagination](https://docs.influxdata.com/influxdb/latest/api/#tag/Pagination).
        :param int limit: Limits the number of records returned. Default is `20`.
        :return: Checks
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_checks_prepare(org_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/checks', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Checks',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _get_checks_prepare(self, org_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['org_id', 'zap_trace_span', 'offset', 'limit']  # noqa: E501
        self._check_operation_params('get_checks', all_params, local_var_params)
        # verify the required parameter 'org_id' is set
        if ('org_id' not in local_var_params or
                local_var_params['org_id'] is None):
            raise ValueError("Missing the required parameter `org_id` when calling `get_checks`")  # noqa: E501

        if 'offset' in local_var_params and local_var_params['offset'] < 0:  # noqa: E501
            raise ValueError("Invalid value for parameter `offset` when calling `get_checks`, must be a value greater than or equal to `0`")  # noqa: E501
        if 'limit' in local_var_params and local_var_params['limit'] > 100:  # noqa: E501
            raise ValueError("Invalid value for parameter `limit` when calling `get_checks`, must be a value less than or equal to `100`")  # noqa: E501
        if 'limit' in local_var_params and local_var_params['limit'] < 1:  # noqa: E501
            raise ValueError("Invalid value for parameter `limit` when calling `get_checks`, must be a value greater than or equal to `1`")  # noqa: E501
        path_params = {}

        query_params = []
        if 'offset' in local_var_params:
            query_params.append(('offset', local_var_params['offset']))  # noqa: E501
        if 'limit' in local_var_params:
            query_params.append(('limit', local_var_params['limit']))  # noqa: E501
        if 'org_id' in local_var_params:
            query_params.append(('orgID', local_var_params['org_id']))  # noqa: E501

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def get_checks_id(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_checks_id(check_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_checks_id_with_http_info(check_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_checks_id_with_http_info(check_id, **kwargs)  # noqa: E501
            return data

    def get_checks_id_with_http_info(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_checks_id_with_http_info(check_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_checks_id_prepare(check_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/checks/{checkID}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Check',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def get_checks_id_async(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a check.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_checks_id_prepare(check_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/checks/{checkID}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Check',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _get_checks_id_prepare(self, check_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['check_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('get_checks_id', all_params, local_var_params)
        # verify the required parameter 'check_id' is set
        if ('check_id' not in local_var_params or
                local_var_params['check_id'] is None):
            raise ValueError("Missing the required parameter `check_id` when calling `get_checks_id`")  # noqa: E501

        path_params = {}
        if 'check_id' in local_var_params:
            path_params['checkID'] = local_var_params['check_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def get_checks_id_labels(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """List all labels for a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_checks_id_labels(check_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_checks_id_labels_with_http_info(check_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_checks_id_labels_with_http_info(check_id, **kwargs)  # noqa: E501
            return data

    def get_checks_id_labels_with_http_info(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """List all labels for a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_checks_id_labels_with_http_info(check_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_checks_id_labels_prepare(check_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/checks/{checkID}/labels', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelsResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def get_checks_id_labels_async(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """List all labels for a check.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_checks_id_labels_prepare(check_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/checks/{checkID}/labels', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelsResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _get_checks_id_labels_prepare(self, check_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['check_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('get_checks_id_labels', all_params, local_var_params)
        # verify the required parameter 'check_id' is set
        if ('check_id' not in local_var_params or
                local_var_params['check_id'] is None):
            raise ValueError("Missing the required parameter `check_id` when calling `get_checks_id_labels`")  # noqa: E501

        path_params = {}
        if 'check_id' in local_var_params:
            path_params['checkID'] = local_var_params['check_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def get_checks_id_query(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a check query.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_checks_id_query(check_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: FluxResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_checks_id_query_with_http_info(check_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_checks_id_query_with_http_info(check_id, **kwargs)  # noqa: E501
            return data

    def get_checks_id_query_with_http_info(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a check query.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_checks_id_query_with_http_info(check_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: FluxResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_checks_id_query_prepare(check_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/checks/{checkID}/query', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='FluxResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def get_checks_id_query_async(self, check_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a check query.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: FluxResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_checks_id_query_prepare(check_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/checks/{checkID}/query', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='FluxResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _get_checks_id_query_prepare(self, check_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['check_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('get_checks_id_query', all_params, local_var_params)
        # verify the required parameter 'check_id' is set
        if ('check_id' not in local_var_params or
                local_var_params['check_id'] is None):
            raise ValueError("Missing the required parameter `check_id` when calling `get_checks_id_query`")  # noqa: E501

        path_params = {}
        if 'check_id' in local_var_params:
            path_params['checkID'] = local_var_params['check_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def patch_checks_id(self, check_id, check_patch, **kwargs):  # noqa: E501,D401,D403
        """Update a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_checks_id(check_id, check_patch, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param CheckPatch check_patch: Check update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.patch_checks_id_with_http_info(check_id, check_patch, **kwargs)  # noqa: E501
        else:
            (data) = self.patch_checks_id_with_http_info(check_id, check_patch, **kwargs)  # noqa: E501
            return data

    def patch_checks_id_with_http_info(self, check_id, check_patch, **kwargs):  # noqa: E501,D401,D403
        """Update a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_checks_id_with_http_info(check_id, check_patch, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param CheckPatch check_patch: Check update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._patch_checks_id_prepare(check_id, check_patch, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/checks/{checkID}', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Check',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def patch_checks_id_async(self, check_id, check_patch, **kwargs):  # noqa: E501,D401,D403
        """Update a check.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param CheckPatch check_patch: Check update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._patch_checks_id_prepare(check_id, check_patch, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/checks/{checkID}', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Check',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _patch_checks_id_prepare(self, check_id, check_patch, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['check_id', 'check_patch', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('patch_checks_id', all_params, local_var_params)
        # verify the required parameter 'check_id' is set
        if ('check_id' not in local_var_params or
                local_var_params['check_id'] is None):
            raise ValueError("Missing the required parameter `check_id` when calling `patch_checks_id`")  # noqa: E501
        # verify the required parameter 'check_patch' is set
        if ('check_patch' not in local_var_params or
                local_var_params['check_patch'] is None):
            raise ValueError("Missing the required parameter `check_patch` when calling `patch_checks_id`")  # noqa: E501

        path_params = {}
        if 'check_id' in local_var_params:
            path_params['checkID'] = local_var_params['check_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'check_patch' in local_var_params:
            body_params = local_var_params['check_patch']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def post_checks_id_labels(self, check_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        """Add a label to a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_checks_id_labels(check_id, label_mapping, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param LabelMapping label_mapping: Label to add (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_checks_id_labels_with_http_info(check_id, label_mapping, **kwargs)  # noqa: E501
        else:
            (data) = self.post_checks_id_labels_with_http_info(check_id, label_mapping, **kwargs)  # noqa: E501
            return data

    def post_checks_id_labels_with_http_info(self, check_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        """Add a label to a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_checks_id_labels_with_http_info(check_id, label_mapping, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param LabelMapping label_mapping: Label to add (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_checks_id_labels_prepare(check_id, label_mapping, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/checks/{checkID}/labels', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def post_checks_id_labels_async(self, check_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        """Add a label to a check.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param LabelMapping label_mapping: Label to add (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_checks_id_labels_prepare(check_id, label_mapping, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/checks/{checkID}/labels', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _post_checks_id_labels_prepare(self, check_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['check_id', 'label_mapping', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('post_checks_id_labels', all_params, local_var_params)
        # verify the required parameter 'check_id' is set
        if ('check_id' not in local_var_params or
                local_var_params['check_id'] is None):
            raise ValueError("Missing the required parameter `check_id` when calling `post_checks_id_labels`")  # noqa: E501
        # verify the required parameter 'label_mapping' is set
        if ('label_mapping' not in local_var_params or
                local_var_params['label_mapping'] is None):
            raise ValueError("Missing the required parameter `label_mapping` when calling `post_checks_id_labels`")  # noqa: E501

        path_params = {}
        if 'check_id' in local_var_params:
            path_params['checkID'] = local_var_params['check_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'label_mapping' in local_var_params:
            body_params = local_var_params['label_mapping']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def put_checks_id(self, check_id, check, **kwargs):  # noqa: E501,D401,D403
        """Update a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_checks_id(check_id, check, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param Check check: Check update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.put_checks_id_with_http_info(check_id, check, **kwargs)  # noqa: E501
        else:
            (data) = self.put_checks_id_with_http_info(check_id, check, **kwargs)  # noqa: E501
            return data

    def put_checks_id_with_http_info(self, check_id, check, **kwargs):  # noqa: E501,D401,D403
        """Update a check.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_checks_id_with_http_info(check_id, check, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param Check check: Check update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._put_checks_id_prepare(check_id, check, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/checks/{checkID}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Check',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def put_checks_id_async(self, check_id, check, **kwargs):  # noqa: E501,D401,D403
        """Update a check.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str check_id: The check ID. (required)
        :param Check check: Check update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Check
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._put_checks_id_prepare(check_id, check, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/checks/{checkID}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Check',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _put_checks_id_prepare(self, check_id, check, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['check_id', 'check', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('put_checks_id', all_params, local_var_params)
        # verify the required parameter 'check_id' is set
        if ('check_id' not in local_var_params or
                local_var_params['check_id'] is None):
            raise ValueError("Missing the required parameter `check_id` when calling `put_checks_id`")  # noqa: E501
        # verify the required parameter 'check' is set
        if ('check' not in local_var_params or
                local_var_params['check'] is None):
            raise ValueError("Missing the required parameter `check` when calling `put_checks_id`")  # noqa: E501

        path_params = {}
        if 'check_id' in local_var_params:
            path_params['checkID'] = local_var_params['check_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'check' in local_var_params:
            body_params = local_var_params['check']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params
